<?php

namespace App\Controller;

use App\Entity\CaseLocation;
use App\Entity\Location;
use App\Entity\MemberCase;
use App\Entity\Messages;
use App\Entity\ReferralSource;
use App\Entity\User;
use App\Entity\UserCase;
use App\Factory\MessageFactory;
use App\Form\LocationFormType;
use App\Form\MemberCaseFormType;
use App\Form\UserCaseFormType;
use App\Libs\Breadcrumb;
use App\Libs\Libs;
use App\Libs\NavList;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Security\Http\Attribute\CurrentUser;

class CaseController extends AbstractController
{
    /**
     * Variable to store unread notification messages
     *
     * @var array
     */
    private array $msgs;

    private int $notificationCount;

    public function __construct(
        private EntityManagerInterface $entityManager,
        private array $navLinks = []
    ) {
        $this->navLinks = NavList::LIST;
        $this->navLinks['case_list'] = NavList::PRESENT_LINK;
    }

    #[Route('/my-cases', name: 'app_my_cases')]
    public function myCases(#[CurrentUser()] User $user, Request $request): Response
    {
        $this->navLinks['my_cases'] = NavList::PRESENT_LINK;
        $this->navLinks['case_list'] = 'nav-link text-dark';

        $ucs = $this->entityManager->getRepository(UserCase::class)->findBy(['user' => $user]);
        $this->msgs = $this->entityManager->getRepository(Messages::class)->getUnreadMessages($user);
        $this->notificationCount = $this->entityManager->getRepository(Messages::class)->getUnreadMessageCount($user);

        $cases = [];
        foreach ($ucs as $uc) {
            /** @var UserCase $uc */
            $cases[] = $uc->getMemberCase();
        }

        return $this->render(
            'internal/cases/my-cases.html.twig',
            array_merge(
                $this->navLinks,
                [
                    'breadcrumbs' => [
                        (
                            strpos($request->server->get('HTTP_REFERER'), 'list-cases') !== false
                                ? new Breadcrumb($this->generateUrl('app_list_cases'), 'List Cases')
                                : new Breadcrumb($this->generateUrl('app_my_cases'), 'My Cases')
                        ),
                    ],
                    'notifications' => $this->msgs,
                    'cases' => $cases,
                    'notificationCount' => $this->notificationCount,
                ]
            )
        );
    }

    #[Route('/list-cases', name: 'app_list_cases')]
    public function listCases(#[CurrentUser()] User $user): Response
    {
        $this->denyAccessUnlessGranted('ROLE_ADMIN');

        $cases = $this->entityManager->getRepository(MemberCase::class)->findAll();
        $workers = $this->entityManager->getRepository(User::class)->getCaseWorkers();
        $this->msgs = $this->entityManager->getRepository(Messages::class)->getUnreadMessages($user);
        $this->notificationCount = $this->entityManager->getRepository(Messages::class)->getUnreadMessageCount($user);

        return $this->render(
            'internal/admin/cases/list-cases.html.twig',
            array_merge(
                $this->navLinks,
                [
                    'breadcrumbs' => [
                        new Breadcrumb($this->generateUrl('app_list_cases'), 'List Cases')
                    ],
                    'notifications' => $this->msgs,
                    'cases' => $cases,
                    'workers' => $workers,
                    'notificationCount' => $this->notificationCount,
                ]
            )
        );
    }

    #[Route('/case/{caseId}', name: 'app_view_case')]
    public function showCase(Request $request, string $caseId): Response
    {
        if (!$this->isGranted('IS_AUTHENTICATED_FULLY')) {
            return $this->redirectToRoute('app_login');
        }

        $case = $this->entityManager->getRepository(MemberCase::class)->find($caseId);
        $sources = $this->entityManager->getRepository(ReferralSource::class)->findAll();

        $this->msgs = $this->entityManager->getRepository(Messages::class)->getUnreadMessages($this->getUser());
        $this->notificationCount = $this->entityManager->getRepository(Messages::class)->getUnreadMessageCount($this->getUser());

        return $this->render(
            'internal/cases/view-case.html.twig',
            array_merge(
                $this->navLinks,
                [
                    'case' => $case,
                    'sources' => $sources,
                    'breadcrumbs' => [
                        new Breadcrumb($this->generateUrl('app_list_cases'), 'List Cases'),
                        new Breadcrumb($this->generateUrl('app_view_case', ['caseId' => $case->getId()]), 'View Case')
                    ],
                    'notifications' => $this->msgs,
                    'notificationCount' => $this->notificationCount,
                ]
            )
        );
    }

    #[Route('/add-case', name: 'app_add_case')]
    public function addCase(Request $request, #[CurrentUser()] User $admin): Response
    {
        $this->denyAccessUnlessGranted('ROLE_ADMIN');
        $this->msgs = $this->entityManager->getRepository(Messages::class)->getUnreadMessages($admin);
        $this->notificationCount = $this->entityManager->getRepository(Messages::class)->getUnreadMessageCount($admin);

        $case = new MemberCase();
        $form = $this->createForm(MemberCaseFormType::class, $case);

        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $case = $form->getData();

            $this->entityManager->persist($case);
            $this->entityManager->flush();

            $this->addFlash('success', 'Case added successfully');

            return $this->redirectToRoute('app_list_cases');
        }

        return $this->render(
            'internal/admin/cases/add-case.html.twig',
            array_merge(
                $this->navLinks,
                [
                    'title' => 'Add Case',
                    'breadcrumbs' => [
                        new Breadcrumb($this->generateUrl('app_list_cases'), 'List Cases'),
                        new Breadcrumb($this->generateUrl('app_add_case'), 'Add Case')
                    ],
                    'notifications' => $this->msgs,
                    'notificationCount' => $this->notificationCount,
                    'form' => $form,
                    'sources' => $this->entityManager->getRepository(ReferralSource::class)->retrieveOrderedList(),
                ]
            )
        );
    }

    #[Route('/edit-case/{id}', name: 'app_edit_case')]
    public function editCase(Request $request, #[CurrentUser()] User $admin, string $id): Response
    {
        $this->denyAccessUnlessGranted('ROLE_ADMIN');
        $this->msgs = $this->entityManager->getRepository(Messages::class)->getUnreadMessages($admin);
        $this->notificationCount = $this->entityManager->getRepository(Messages::class)->getUnreadMessageCount($admin);

        $case = $this->entityManager->getRepository(MemberCase::class)->find($id);
        $form = $this->createForm(MemberCaseFormType::class, $case);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $case = $form->getData();

            $this->entityManager->flush();

            $this->addFlash('success', 'Case updated successfully');

            return $this->redirectToRoute('app_list_cases');
        }

        return $this->render(
            'internal/admin/cases/edit-case.html.twig',
            array_merge(
                $this->navLinks,
                [
                    'title' => 'Edit Case',
                    'breadcrumbs' => [
                        new Breadcrumb($this->generateUrl('app_list_cases'), 'List Cases'),
                        new Breadcrumb($this->generateUrl('app_edit_case', ['id' => $id]), 'Edit Case')
                    ],
                    'notifications' => $this->msgs,
                    'notificationCount' => $this->notificationCount,
                    'form' => $form,
                    'case' => $case,
                    'sources' => $this->entityManager->getRepository(ReferralSource::class)->retrieveOrderedList(),
                ]
            )
        );
    }

    #[Route('/assign-case/{id}', name: 'app_assign_case')]
    public function assignCase(string $id, Request $request, #[CurrentUser()] User $admin): Response
    {
        $this->denyAccessUnlessGranted('ROLE_ADMIN');
        $this->msgs = $this->entityManager->getRepository(Messages::class)->getUnreadMessages($admin);
        $this->notificationCount = $this->entityManager->getRepository(Messages::class)->getUnreadMessageCount($admin);

        $caseWorkers = $this->entityManager->getRepository(User::class)->getCaseWorkers();
        $case = $this->entityManager->getRepository(MemberCase::class)->find($id);
        $prevUc = $this->entityManager->getRepository(UserCase::class)->findBy(['memberCase' => $case]);

        $uc = new UserCase();
        $form = $this->createForm(UserCaseFormType::class, $uc);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $user = $form->get('user')->getData();
            $uc = $form->getData();
            $uc->setMemberCase($case);
            $uc->setUser($user);

            if (count($prevUc) > 0) {
                $this->entityManager->remove($prevUc[0]);
                $this->entityManager->flush();
            }

            $msg = MessageFactory::createNewCase($admin, $user);

            $this->entityManager->persist($uc);
            $this->entityManager->persist($msg);
            $this->entityManager->flush();

            $this->addFlash('success', 'Case assigned successfully');

            return $this->redirectToRoute('app_list_cases');
        }

        return $this->render(
            'internal/admin/cases/assign-case.html.twig',
            array_merge(
                $this->navLinks,
                [
                    'title' => 'Assign Case',
                    'breadcrumbs' => [
                        new Breadcrumb($this->generateUrl('app_list_cases'), 'List Cases'),
                        new Breadcrumb($this->generateUrl('app_assign_case', ['id' => $id]), 'Assign User')
                    ],
                    'notifications' => $this->msgs,
                    'notificationCount' => $this->notificationCount,
                    'case' => $case,
                    'form' => $form,
                    'id' => $id,
                    'caseWorkers' => $caseWorkers,
                    'assignedWorkerId' => (count($case->getUserCases()) > 0 ? $case->getUserCases()->first()->getUser()->getId() : null),
                ]
            )
        );
    }

    #[Route('/case-notes/{id}', name: 'app_case_notes')]
    public function showCaseNotes(Request $request, string $id): Response
    {
        $case = $this->entityManager->getRepository(MemberCase::class)->find($id);
        return new Response();
    }

    #[Route('/addresses/list', name: 'app_list_case_addresses')]
    public function listCaseAddresses(Request $request, #[CurrentUser()] User $user): Response
    {
        $this->navLinks['case_itinerary'] = NavList::PRESENT_LINK;
        $this->navLinks['case_list'] = 'nav-link text-dark';

        $addresses = $this->entityManager->getRepository(Location::class)->getUserLocations($user);
        $this->msgs = $this->entityManager->getRepository(Messages::class)->getUnreadMessages($user);
        $this->notificationCount = $this->entityManager->getRepository(Messages::class)->getUnreadMessageCount($user);

        $ucs = $this->entityManager->getRepository(UserCase::class)->findBy(['user' => $user]);
        $cases = [];
        foreach ($ucs as $uc) {
            /** @var UserCase $uc */
            $cases[] = $uc->getMemberCase();
        }

        return $this->render(
            'internal/cases/addresses/list-case-addresses.html.twig',
            array_merge(
                $this->navLinks,
                [
                    'title' => 'List Case Addresses',
                    'breadcrumbs' => [
                        new Breadcrumb($this->generateUrl('app_my_cases'), 'My Cases'),
                        new Breadcrumb($this->generateUrl('app_list_case_addresses'), 'List Case Addresses')
                    ],
                    'notifications' => $this->msgs,
                    'notificationCount' => $this->notificationCount,
                    'addresses' => $addresses,
                    'cases' => $cases,
                ]
            )
        );
    }

    #[Route('/addresses/add', name: 'app_case_add_address')]
    public function addCaseAddress(Request $request, #[CurrentUser()] User $user): Response
    {
        $this->navLinks['case_itinerary'] = NavList::PRESENT_LINK;
        $this->navLinks['case_list'] = 'nav-link text-dark';

        $this->msgs = $this->entityManager->getRepository(Messages::class)->getUnreadMessages($user);
        $this->notificationCount = $this->entityManager->getRepository(Messages::class)->getUnreadMessageCount($user);
        $ucs = $this->entityManager->getRepository(UserCase::class)->findBy(['user' => $user]);

        $cases = [];
        foreach ($ucs as $uc) {
            /** @var UserCase $uc */
            $cases[] = $uc->getMemberCase();
        }

        $address = new Location();
        $form = $this->createForm(LocationFormType::class, $address);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            /** @var Location $address */
            $address = $form->getData();

            foreach ($request->get('cases') as $caseId) {
                $case = $this->entityManager->getRepository(MemberCase::class)->find($caseId);

                $cl = new CaseLocation();
                $cl->setMemberCase($case)
                   ->setLocation($address);

                $this->entityManager->persist($cl);
            }

            list($lat, $lon) = Libs::getLatLonFromGeoapify((string) $address);
            $address->setLat($lat)
                    ->setLon($lon);

            //dd($address);
            $this->entityManager->persist($address);
            $this->entityManager->flush();

            $this->addFlash('success', 'Address added successfully');

            return $this->redirectToRoute('app_list_case_addresses');
        }

        return $this->render(
            'internal/cases/addresses/add-case-address.html.twig',
            array_merge(
                $this->navLinks,
                [
                    'title' => 'Add Case Address',
                    'breadcrumbs' => [
                        new Breadcrumb($this->generateUrl('app_list_case_addresses'), 'List Addresses'),
                        new Breadcrumb($this->generateUrl('app_case_add_address'), 'Add Case Address')
                    ],
                    'notifications' => $this->msgs,
                    'notificationCount' => $this->notificationCount,
                    'form' => $form,
                    'cases' => $cases,
                ]
            )
        );
    }

    #[Route('/addresses/edit/{id}', name: 'app_case_edit_address')]
    public function editCaseAddress(Request $request, string $id): Response
    {
        $this->navLinks['case_itinerary'] = NavList::PRESENT_LINK;
        $this->navLinks['case_list'] = 'nav-link text-dark';

        $this->msgs = $this->entityManager->getRepository(Messages::class)->getUnreadMessages($this->getUser());
        $this->notificationCount = $this->entityManager->getRepository(Messages::class)->getUnreadMessageCount($this->getUser());

        $ucs = $this->entityManager->getRepository(UserCase::class)->findBy(['user' => $this->getUser()]);
        $lcs = $this->entityManager->getRepository(CaseLocation::class)->findBy(['location' => $id]);

        $inCases = [];
        foreach ($lcs as $lc) {
            $inCases[] = $lc->getMemberCase()->getId()->toString();
        }

        $cases = [];
        foreach ($ucs as $uc) {
            /** @var UserCase $uc */
            $cases[] = $uc->getMemberCase();
        }
        $location = $this->entityManager->getRepository(Location::class)->find($id);
        $form = $this->createForm(LocationFormType::class, $location);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            list($lat, $lon) = Libs::getLatLonFromGeoapify((string) $location);
            $location->setLat($lat)
                     ->setLon($lon);

            $this->entityManager->flush();

            $this->addFlash('success', 'Address updated successfully');

            return $this->redirectToRoute('app_list_case_addresses');
        }

        return $this->render(
            'internal/cases/addresses/edit-case-address.html.twig',
            array_merge(
                $this->navLinks,
                [
                    'title' => 'Edit Case Address',
                    'breadcrumbs' => [
                        new Breadcrumb($this->generateUrl('app_list_case_addresses'), 'List Case Addresses'),
                        new Breadcrumb($this->generateUrl('app_case_edit_address', ['id' => $id]), 'Edit Case Address')
                    ],
                    'location' => $location,
                    'cases' => $cases,
                    'inCases' => $inCases,
                    'form' => $form,
                    'notifications' => $this->msgs,
                    'notificationCount' => $this->notificationCount,
                ]
            )
        );
    }

    #[Route('/api/filter-address-by-case/{caseId}', name: 'ajax_filter_address_by_case')]
    public function filterAddressByCase(string $caseId, Request $request): Response
    {
        $case = $this->entityManager->getRepository(MemberCase::class)->find($caseId);
        $addresses = $this->entityManager->getRepository(Location::class)->getLocationsByCase($case);

        return $this->json($addresses);
    }

    #[Route('/api/case/{caseId}/user/{userId}', name: 'ajax_case_user_level_check')]
    public function checkUserCaseLevel(string $caseId, string $userId) : Response
    {
        $ret = true;

        $user = $this->entityManager->getRepository(User::class)->find($userId);
        $case = $this->entityManager->getRepository(MemberCase::class)->find($caseId);

        $res = UserCase::checkLevel($user, $case);

        if (!$res) {
            $ret = [
                'userLevel' => ucwords(str_replace('_', ' ', strtolower($user->getLevel()->name))),
                'caseLevel' => ucwords(str_replace('_', ' ', strtolower($case->getLevel()->name))),
            ];
        }

        return $this->json($ret);
    }

    #[Route('/api/filter-cases-by-user', name: 'ajax_filter_cases_by_user')]
    public function filterCasesByUser(Request $request): Response
    {
        /** @var User $user */
        $user = $this->entityManager->getRepository(User::class)->find($request->query->get('userId'));
        /** @var UserCase[] $ucs */
        $ucs = $this->entityManager->getRepository(UserCase::class)->findBy(['user' => $user]);

        $ret = [];
        foreach ($ucs as $uc) {
            $ret[] = $uc->getMemberCase();
        }

        dump($ret);
        die;
        return $this->json($ret);
    }
}
